﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.ServiceModel;
using System.ServiceModel.Description;
using System.Diagnostics;

namespace ClientServer_Client.Client
{
    /// <summary>
    /// https://msdn.microsoft.com/de-de/library/1fk1t1t0.aspx - Konfigurationsdateischema für .NET Framework
    /// 
    /// https://msdn.microsoft.com/de-de/library/ms734663%28v=vs.110%29.aspx - app.config
    /// 
    /// https://msdn.microsoft.com/de-de/library/ms733932%28v=vs.110%29.aspx - LESEN!!!
    /// 
    /// 
    /// 
    /// Step by step - On client side:
    /// 
    /// - create binding - type of binding on server sand on client has to be the same (here - pipe): 
    /// 
    ///     var binding = new NetNamedPipeBinding(NetNamedPipeSecurityMode.None);
    /// 
    /// - create an instance of the proxy class, generated with SvcUtil.exe:
    /// 
    ///     m_ServerStub = new ServerStubClient(binding, new EndpointAddress("net.pipe://localhost/TestClientServer.Service/TestClientServerService "));
    ///     
    ///         Info about string:
    ///             - TestClientServer.Service = server.interface-class.[ServiceContract(Namespace = "TestClientServer.Service")]
    ///             - TestClientServerService = server.m_Host.AddServiceEndpoint(typeof(IServerStub), binding, "TestClientServerService");
    ///             - other way to found it - we can see this info in output.config:
    ///                 <endpoint address="net.pipe://localhost/TestClientServer.Service/TestClientServerService"
    /// 
    /// </summary>
    public class ClientForProxy
    {
        ///////////////////////////////////////////////

        #region members

        const string c_serverName = @"..\..\..\..\ClientServer_Server\ClientServer_Server\bin\Debug\ClientServer_Server.exe";

        ServerStubClient m_ServerStub; // = new ServerStubClient("net.pipe://localhost/TestClientServer.Service");

        Process m_ServerProcess;

        #endregion

        ///////////////////////////////////////////////

        #region start / stop server

        public bool StartServer()
        {
            try
            {
                m_ServerProcess = System.Diagnostics.Process.Start(c_serverName);
                if (null == m_ServerProcess)
                    throw new InvalidOperationException("server is not running");

                // string - from generated output.config (it is different with the string, defined in server - code)
                var binding = new NetNamedPipeBinding(NetNamedPipeSecurityMode.None);
                m_ServerStub = new ServerStubClient(binding, new EndpointAddress("net.pipe://localhost/TestClientServer.Service/TestClientServerService"));
                return true;
            }
            catch (System.IO.IOException e)
            {
                System.Windows.Forms.MessageBox.Show("ERROR", c_serverName + " could not be started. Error = " + e.Message);
                return false;
            }
        }

        public bool StopServer()
        {
            try
            {
                m_ServerProcess.Kill();
                return true; 
            }
            catch (System.IO.IOException e)
            {
                System.Windows.Forms.MessageBox.Show("ERROR", c_serverName + " could not be closed. Error = " + e.Message);
                return false;
            }
        }
        
        #endregion

        ///////////////////////////////////////////////

        #region call server members

        public void CallServerFunc()
        {
            string answer = m_ServerStub.ShowMsgBox("test1", "do you see it?");
            System.Windows.Forms.MessageBox.Show(answer, "CallServerFunc() *** Result - OK ");
        }

        public void CallServerFunc_LoadDll()
        {
            m_ServerStub.CreateCommunicationServerComInstance("dummy_ProgID");
            System.Windows.Forms.MessageBox.Show("CallServerFunc_LoadDll() *** Result - OK ");
        }
        
        #endregion

        ///////////////////////////////////////////////
    }


}
