﻿

////////////////////////////////////
// IMPORTANT!!! this constant will be used only once by creating of the proxy - class for our client
////////////////////////////////////

//#define PrepareClient



using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

//using System.StubHelpers;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.InteropServices;


using ClientServer_Server.ServerStub.Interfaces;




// How to:
//
// 1) add reference: C:\Program Files (x86)\Reference Assemblies\Microsoft\Framework\v3.0\System.ServiceModel.dll - 
//              - it will be needed for using System.ServiceModel - namespace
//
// 2) to create a stub for client we need to do following:
//      - create output directory (for example: d:\test\client)
//      - compile and then start this application AS ADMIN (!!!)
//      - Utility to generate client code: C:\Program Files (x86)\Microsoft SDKs\Windows\v7.0A\bin\NETFX 4.0 Tools\SvcUtil.exe
//              see: https://msdn.microsoft.com/de-de/library/aa347733%28v=vs.110%29.aspx        
//      - call it in cmd.exe:
//              C:\Program Files (x86)\Microsoft SDKs\Windows\v7.0A\Bin\NETFX 4.0 Tools>SvcUtil.exe http://localhost:8123/TestClientServer.Service /language:c# /directory:d:\test\client\server.cs
//      - check created file
//      - stop this exe (input a number != 0 in the command-window)
//      - in both files comment a line #define PrepareClient
//      - copy both (.cs and .config) generated files (to found - under d:\test\client) to client-solution
// 
// 3) Used strings on server:
//
//      3.1) Interface->[ServiceContract(Namespace = "TestClientServer.Service")] will be used in:
//              - new ServiceHost(this, new Uri("net.pipe://localhost/TestClientServer.Service"));
//              - metaBehaviour.HttpGetUrl = new Uri("http://localhost:8123/TestClientServer.Service");          
//
//      3.2) m_Host.AddServiceEndpoint(typeof(IServerStub), binding, "TestClientServerService") will be used in:
//
// 4) use the same strings on the client side:
//
//      we need to create a binding to server. ServerStubClient - proxy-class, created with SvcUtil.exe.
//
//      m_ServerStub = new ServerStubClient(binding, new EndpointAddress("net.pipe://localhost/TestClientServer.Service/TestClientServerService "));
//
//          where: TestClientServer.Service = 3.1)
//                  TestClientServerService = 3.2)
// 
namespace ClientServer_Server.ServerStubService
{

    /// <summary>
    /// https://msdn.microsoft.com/de-de/library/system.servicemodel.netnamedpipebinding%28v=vs.110%29.aspx - pipe binding
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class ServerStub : IServerStub
    {
        //////////////////////////////////////////

        #region members, ole32.dll

        private readonly ServiceHost m_Host;

        private bool m_ServerRunning = false;
        public bool ServerRunning
        {
            get { return m_ServerRunning; }
        }

        public const string c_Default = "not called";
        public const string c_Ok = "Ok - Button was selected";
        public const string c_Cancel = "Cancel - Button was selected";

        ///////////////////////////////////////


        private IFdtCommunication m_communicationServer;

        ///////////////////////////////////////

        #region dllImport (ole32.dll)

        [DllImport("ole32.dll")]
        private static extern int CLSIDFromProgID([MarshalAs(UnmanagedType.LPWStr)] string lpszProgID, out Guid pclsid);

        #endregion


        #endregion

        //////////////////////////////////////////


        #region ctor

        public ServerStub()
        {
            m_Host = new ServiceHost(this, new Uri("net.pipe://localhost/TestClientServer.Service"));
        }

        #endregion

        //////////////////////////////////////////

        #region IServerStub
        

        /// <summary>
        /// implementation: simple test 
        /// </summary>
        /// <param name="messagePart1"></param>
        /// <param name="messagePart2"></param>
        /// <returns></returns>
        public string ShowMsgBox(string messagePart1, string messagePart2)
        {
            string result = c_Default;

            Console.WriteLine("read this message and answer: " + messagePart1 + " *** " + messagePart2);
            Console.WriteLine(System.Environment.NewLine);
            Console.WriteLine("get 1 for " + c_Ok);
            Console.WriteLine(System.Environment.NewLine);
            Console.WriteLine("OR: get 2 for " + c_Cancel);

            string answer = Console.ReadLine();
            switch (answer)
            {
                case "1":
                    result = c_Ok;
                    break;

                case "2":
                    result = c_Cancel;
                    break;

                default:
                    result = c_Default;
                    break;
            }

            return result;
        }


        /// <summary>
        /// implementation: load dll
        /// </summary>
        /// <param name="progID"></param>
        public void CreateCommunicationServerComInstance(string progID)
        {
            try
            {
                // TODO!!! without valid progID this code will be throw exception, that is why only for the test - message:
                Console.WriteLine("SERVER *** CreateCommunicationServerComInstance() was called. OK!!!");


                Guid guid;
                //if (CLSIDFromProgID(progID, out guid) == 0)
                //{
                //    try
                //    {
                //        m_communicationServer = CreateCommunicationInstance(guid);

                //        if (m_communicationServer != null)
                //        {
                //            System.Console.WriteLine(" ************** Server was founded!!! ************** ");
                //        }
                //    }
                //    catch (Exception e)
                //    {
                //        throw new InvalidCastException("Couldn't cast COM Object cs Client" + e.Message);
                //    }
                //}
                //else
                //{
                //    throw new InvalidComObjectException("Couldn't get CLSID from ProgID");
                //}
            }
            catch (Exception e)
            {
                System.Console.WriteLine("Exception occured: " + e.Message);
            }
        }


        #endregion

        //////////////////////////////////////////
        #region private methods

       
        #region LoadDll

        /// <summary>
        /// load dll
        /// </summary>
        /// <param name="guid"></param>
        /// <returns></returns>
        private IFdtCommunication CreateCommunicationInstance(Guid guid)
        {
            Object communicationServerComInstance = Activator.CreateInstance(Type.GetTypeFromCLSID(guid));
            return ((communicationServerComInstance) as IFdtCommunication);
        }

        #endregion LoadDll

        ///////////////////////////////////////

        #region start / stop
        
        /// <summary>
        /// start server
        /// </summary>
        internal void StartServer()
        {
            try
            {
                var binding = new NetNamedPipeBinding(NetNamedPipeSecurityMode.None);
                m_Host.AddServiceEndpoint(typeof(IServerStub), binding, "TestClientServerService");

#if PrepareClient
                var metaBehaviour = new ServiceMetadataBehavior();
                metaBehaviour.HttpGetEnabled = true;
                metaBehaviour.HttpGetUrl = new Uri("http://localhost:8123/TestClientServer.Service");

                m_Host.Description.Behaviors.Add(metaBehaviour);
#endif

                m_Host.Open();
                m_ServerRunning = true;
            }
            catch (CommunicationException e)
            {
                m_Host.Abort();
                m_ServerRunning = false;
                throw;
            }

        }


        /// <summary>
        ///  stop server
        /// </summary>
        internal void StopServer()
        {
            try
            {
                m_Host.Close();
                m_ServerRunning = false;
            }
            catch (CommunicationException)
            {
                m_Host.Abort();
                m_ServerRunning = false;
                throw;
            }
        }

        #endregion


        ///////////////////////////////////////

        #endregion private methods


    }
}
